
clear all
close all

%% initialization
dt = 50;
TIME = 500;                         %350 when testing completion and conflicting input
delta_w = 0.005;                    %learning coefficient
delta_w_heb = 0.05;                 %learning coefficient

dim1=8;                             %number of units
dim2=2;                             %number of units
dim3=1;                             %number of units
input = zeros(TIME,dim1);           %vector of input
reward= zeros(TIME,dim3);           %vector of reward

%initialization of two hypothetical neural structures 
n1_potential = zeros(TIME,dim1);    %all nodes in structure 1: action potential across time
n1_activity  = zeros(TIME,dim1);    %all nodes in structure 1: activity across time
n2_potential = zeros(TIME,dim2);    %all nodes in structure 2: action potential across time
n2_activity  = zeros(TIME,dim2);    %all nodes in structure 2: activity across time

da_potential = zeros(TIME,dim3);    %all nodes in structure 2: action potential across time
da_activity  = zeros(TIME,dim3);    %all nodes in structure 2: activity across time


%weights
input2n1 = eye(dim1);               %matrix of all weights from input to n1
input2n2 = zeros(dim1,dim2);        %matrix of all weights from input to n2

mat_w1    =zeros(dim1, dim1);       %matrix of all weights within n1
mat_w1_store=zeros(TIME,dim1*dim1); %matrix used to store changes in weights of mat_w1
mat_w_n1n2=zeros(dim1, dim2);       %matrix of all weights from n1 to n2
mat_w_n1n2_store=...
    zeros(TIME,dim1*dim2);          %matrix used to store changes in weights of mat_w_n1n2

mat_w2=ones(dim2, dim2)*(-1) + ...  
    eye(dim2)*1.5;                  %matrix of all weights within n2: lateral inhibition
mat_w2_store=zeros(TIME,dim2*dim2); %matrix used to store changes in weights of mat_w2

mat_w_Rda =ones(dim3,dim3)*3;         %matrix of all weights from reward to dopamine units
mat_w3    =zeros(dim3, dim3);       %matrix of all weights within da structure


%vector of decays
decay1=300;
decay2=150;
v_dec1=ones(1,dim1)*decay1;
v_dec2=ones(1,dim2)*decay1;
v_dec3=ones(1,dim3)*decay2;

%vector of baselines
v_bl1=zeros(1,dim1);
v_bl2=ones(1,dim2)*0.2;
v_bl3=zeros(1,dim3);

%vector of threshold
v_th1=zeros(1,dim1);
v_th2=zeros(1,dim2);
v_th3=zeros(1,dim3);

%vector of noise activation
v_noise1=zeros(1,dim1);              %if noise=1 -> bl=bl+-0.1, valid for all units
v_noise2=ones(1,dim2)*10;            %if noise=1 -> bl=bl+-0.1, valid for all units
v_noise3=zeros(1,dim3);              %if noise=1 -> bl=bl+-0.1, valid for all units

%% input variablity
%completion_input
%conflicting_input
RL_input

% step
for t=2:TIME
%% neural activity    

    n1_activity(t,:)=layer_step(t, dt, n1_potential, n1_activity, input(t-1,:)*input2n1, ...
        v_dec1, v_bl1, mat_w1, v_th1, v_noise1);
    
    n2_activity(t,:)=layer_step(t, dt, n2_potential, n2_activity, input(t-1,:)* input2n2 + n1_activity(t-1,:)*mat_w_n1n2,...
        v_dec2, v_bl2, mat_w2, v_th2, v_noise2);    

    da_activity(t,:)=layer_step(t, dt, da_potential, da_activity, reward(t-1,:)*mat_w_Rda,...
        v_dec3, v_bl3, mat_w3, v_th3, v_noise3);
    
    
    %% Check presence of reward
    if sum(input(t,5:8))>=1.5 && n2_activity(t,1)>=0.4 && t<450
        reward(t)=1;
    end
    
%Artificial reset when input=0 for a long time. Used to avoid total interconnection
    if t>50 && sum(input(t,:))==0 && sum(input(t-25,:))==0
        n1_activity(t,:)=zeros(1,dim1);
    elseif reward(t)==1 && reward(t)==reward(t-2)
        %da_activity(t,:)=zeros(1,dim3); 
        reward(t)=0;
    end    
    

%% Autoassociative learning 

%% hebbian rule
for i_row=1:dim1
    for i_col=1:dim1
        if i_row==i_col
        else
            mat_w1(i_row,i_col)=mat_w1(i_row,i_col)+(n1_activity(t,i_row)*n1_activity(t,i_col))*delta_w_heb;
            mat_w1(i_row,i_col)=min(2,mat_w1(i_row,i_col)); %saturation for the weigths of the synapses
        end
    end
end

%% post-synaptic hebbian rule
% for i_row=1:dim1
%     for i_col=1:dim1
%         if i_row==i_col
%         else
%             % mind the threshold of activity for the formula   (yi)*(2*yj-threshold)
%             mat_w1(i_row,i_col)=mat_w1(i_row,i_col)+(n1_activity(t,i_row)*(2*n1_activity(t,i_col)-0.4))*delta_w_heb;
%             mat_w1(i_row,i_col)=min(2,mat_w1(i_row,i_col)); %saturation for the weigths of the synapses
%         end
%     end
% end

%% pre-synaptic hebbian rule
% for i_row=1:dim1
%     for i_col=1:dim1
%         if i_row==i_col
%         else
%            % mind the threshold of activity for the formula        (2*yi-threshold)*(2*yj)
%             mat_w1(i_row,i_col)=mat_w1(i_row,i_col)+((2*n1_activity(t,i_row)-0.4)*n1_activity(t,i_col))*delta_w_heb;
%         end
%     end
% end

%% Dopamine-dependent learning (post-synaptic hebbian rule)
 
for i_row=1:dim1
    for i_col=1:dim2
        if i_row==i_col
        else
            mat_w_n1n2(i_row,i_col)=mat_w_n1n2(i_row,i_col)+(n1_activity(t,i_row)*(n2_activity(t,i_col)-0.3))*(da_activity(t,:));
        end
    end
end

%% store mat_w changes
mat_w1_store(t,:)    = reshape (mat_w1, 1,dim1*dim1);
mat_w_n1n2_store(t,:)= reshape (mat_w_n1n2, 1,dim1*dim2);
end

figure
subplot (4,1,1)
plot (input, '.-');
title ('input','FontWeight','bold', 'FontSize',16)
subplot (4,1,2)
plot (n1_activity, '.-');
title ('n1','FontWeight','bold', 'FontSize',16)
subplot (4,1,3)
plot (n2_activity, '.-');
title ('n2','FontWeight','bold', 'FontSize',16)
subplot (4,1,4)
plot (da_activity, '.-');
title ('Dopamine','FontWeight','bold', 'FontSize',16)

figure
subplot (4,1,1)
imagesc(input');
title ('input','FontWeight','bold', 'FontSize',16)
subplot (4,1,2)
imagesc(n1_activity');
title ('n1','FontWeight','bold', 'FontSize',16)
subplot (4,1,3)
imagesc(n2_activity');
title ('n2','FontWeight','bold', 'FontSize',16)
subplot (4,1,4)
imagesc(da_activity');
title ('Dopamine','FontWeight','bold', 'FontSize',16)

figure; plot(mat_w1_store);
figure; imagesc(mat_w1_store');
figure; plot(mat_w_n1n2_store);
figure; imagesc(mat_w_n1n2_store');