
clear all
close all

%% initialization
dt = 50;
TIME = 500;                         %350 when testing completion and conflicting input
delta_w = 0.005;                    %learning coefficient
delta_w_heb = 0.05;                 %learning coefficient

dim1=3;                             %number of units
dim2=1;
input_n = zeros(TIME,dim1);         %vector of inputs towards neurons
input_da = zeros(TIME,dim2);        %vector of inputs towards dopaminergic units

%initialization of two hypothetical neural structures 
n1_potential = zeros(TIME,dim1);    %all nodes in structure 1: action potential across time
n1_activity  = zeros(TIME,dim1);    %all nodes in structure 1: activity across time

da_potential = zeros(TIME,dim2);    %all nodes in structure 2: action potential across time
da_activity  = zeros(TIME,dim2);    %all nodes in structure 2: activity across time


%weights
input2n1 = eye(dim1);               %matrix of all weights from input to n1
input2da = eye(dim2);               %matrix of all weights from input to DA

mat_w1=ones(dim1)*(-2)+...          %matrix of all weights within n1: 
    eye(dim1)*2;                    %lateral inhibitions and self-excitatory synapses

mat_da=zeros(dim2);                 %matrix of all weights within da

%vector of decays
decay1=300;
decay2=150;
v_dec1=ones(1,dim1)*decay1;
v_dec2=ones(1,dim2)*decay1;


%vector of baselines
v_bl1=zeros(1,dim1);
v_bl2=zeros(1,dim2);

%vector of threshold
v_th1=zeros(1,dim1);
v_th2=zeros(1,dim2);

%vector of noise activation
v_noise1=ones(1,dim1)*5;           %if noise=1 -> bl=bl+-0.1, valid for all units
v_noise2=zeros(1,dim2);             %if noise=1 -> bl=bl+-0.1, valid for all units

%dopamine parameters
epsilon=0.1;                        %signal unaffected by DA presence
lambda=4;                         %signal affected by DA presence

%% input variablity
%completion_input
%conflicting_input
%RL_input
tonic_da_test

% step
for t=2:TIME
%% neural activity    

    n1_activity(t,:)=layer_step(t, dt, n1_potential, n1_activity, (input_n(t-1,:)*input2n1)*(epsilon+lambda*da_activity(t-1,:)), ...
        v_dec1, v_bl1, mat_w1, v_th1, v_noise1);

    da_activity(t,:)=layer_step(t, dt, da_potential, da_activity, input_da(t-1,:)*input2da,...
        v_dec2, v_bl2, mat_da, v_th2, v_noise2);
    
    
end

figure
subplot (3,1,1)
plot (input_n, '.-');
title ('input','FontWeight','bold', 'FontSize',16)
subplot (3,1,2)
plot (n1_activity, '.-');
title ('n1','FontWeight','bold', 'FontSize',16)
subplot (3,1,3)
plot (da_activity*lambda, '.-');
title ('Dopamine','FontWeight','bold', 'FontSize',16)

figure
subplot (3,1,1)
imagesc(input_n');
title ('input','FontWeight','bold', 'FontSize',16)
subplot (3,1,2)
imagesc(n1_activity');
title ('n1','FontWeight','bold', 'FontSize',16)
subplot (3,1,3)
imagesc(da_activity');
title ('Dopamine','FontWeight','bold', 'FontSize',16)